#!/usr/bin/env python
#Copyright (C) 2007 - JamesWP
#Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.og>

import commands
from time import sleep
from commands import getoutput
from os import path
from sys import exit

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "kiba-battery"

# How often should this update? (Must be > 0 to start)
CHECK_INTERVALL = 10000

if getoutput("kiba-settings --get-bool battery_enable") != "TRUE":
	exit()

#Set this if you want to use a custom Theme THEME_DIRECTORY
THEME_DIRECTORY = ""
if THEME_DIRECTORY == "":
	if path.isdir('/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME
	elif path.isdir('/usr/share/kiba-dock/icons/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/usr/share/kiba-dock/icons/' + PLUGIN_NAME
	else:
		print "Kiba-Battery: No theme found, exiting"
		exit()
THEME=THEME_DIRECTORY

class KibaBattery(object):
	icon=THEME + '/0.png'
	description=''
	icon_added=0
	
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(CHECK_INTERVALL, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		self.check_batterie()
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.add_kiba_icon()
			self.set_icon_and_titel()
	
	# Looks at the output from acpi and works out different stuff from it :p
	def check_batterie (self):
		Output = str(commands.getoutput('acpi -b'))
		#FIXME: this probably does not match all languages
		if Output == '' or Output == 'sh: acpi: command not found':
			print 'Kiba-Battery: No Battery Detected, exiting'
			exit()
		Charge = str(Output.split(', ')[1])
		Charge = int(Charge.split('%')[0])
		Remaining = Output.split(', ')
		if len(Remaining) >= 3 :
			Remaining = str(Output.split(', ')[2])
			Remaining = str(Remaining.split(' remaining')[0])
			if isinstance(Remaining, (int, long, float, complex)) :
				Remaining = ('| %s Remaining' % (Remaining))
			else: 
				Remaining = ('| %s' % (Remaining))
		else:
			Remaining = ('')
			
		Name = str(Output.split('     ')[1])
		Name = str(Name.split(':')[0])
		Status = str(Output.split(': ')[1])
		Status = str(Status.split(',')[0])
		
		self.description = ('%s | %s | %s%s Charge %s' % (Name,Status,Charge,'%',Remaining))
		
		if Charge < 100:
			self.icon = THEME + '/100.png'
		if Charge < 76:
			self.icon = THEME + '/75.png'
		if Charge < 51:
			self.icon = THEME + '/50.png'
		if Charge < 26:
			self.icon = THEME + '/25.png'
		if Charge < 1:
			self.icon = THEME + '/0.png'
		if Charge < 0:
			self.icon = THEME + '/none.png'
		
	def set_icon_and_titel(self):
		self.kiba.SetIcon(PLUGIN_NAME, self.icon)
		self.kiba.SetTitel(PLUGIN_NAME, self.description)
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "NULL", self.icon)
		
		self.icon_added=1
		
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
	def close_handler(self, sender=None):
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.add_kiba_icon()
	
	def timeout_func(self):
		if self.icon_added == 0:
			return 1
		
		self.check_batterie()
		self.set_icon_and_titel()
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_battery=KibaBattery()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_battery.kiba.RemoveObject(PLUGIN_NAME)
